from enum import Enum


class PydanticVersion(Enum):
    v1 = 1
    v2 = 2


def get_pydantic_version() -> PydanticVersion:
    try:
        import pydantic  # noqa
    except ModuleNotFoundError:
        raise ModuleNotFoundError("Are you sure you installed Pydantic")

    if pydantic.__version__.startswith("1"):
        return PydanticVersion.v1
    if pydantic.__version__.startswith("2"):
        return PydanticVersion.v2
    raise ModuleNotFoundError("Something went wrong withs Pydantic imports")



from .base_validator import FieldMaskValidator

__all__ = ["CEPValidator"]


class CEPValidator(FieldMaskValidator):
    def __init__(self, cep) -> None:
        self.cep = cep

    def validate_mask(self) -> bool:
        if len(self.cep) == 9:
            return self.cep[5] == "-"

    def validate(self) -> bool:
        cep = self.cep.replace("-", "")

        if len(cep) != 8:
            return False
        return True



from .field_class_errors import get_pydantic_errors_class

__all__ = [
    "FieldTypeError",
    "FieldMaskError",
    "FieldDigitError",
    "FieldInvalidError",
]


PydanticTypeError, PydanticValueError = get_pydantic_errors_class()


class FieldTypeError(PydanticTypeError):
    code = "not_str"
    msg_template = "Input should be a valid string"
    message_template = msg_template


class FieldMaskError(PydanticValueError):
    code = "invalid_mask"
    msg_template = "invalid mask format"
    message_template = msg_template


class FieldDigitError(PydanticValueError):
    code = "not_digits"
    msg_template = "field only accept digits as string"
    message_template = msg_template


class FieldInvalidError(PydanticValueError):
    code = "invalid_data"
    msg_template = "invalid data"
    message_template = msg_template



from typing import Any, Dict, Tuple, Union

from .get_versions import get_pydantic_version

pydantic_version = get_pydantic_version()


class PydanticCustomError(ValueError):
    @property
    def context(self) -> Union[Dict[str, Any], None]:
        ...

    @property
    def type(self) -> str:
        ...

    @property
    def message_template(self) -> str:
        ...

    def message(self) -> str:
        ...


def get_pydantic_errors_class() -> Tuple:
    if pydantic_version.value == 1:
        from pydantic import PydanticTypeError, PydanticValueError
    if pydantic_version.value == 2:

        class PydanticTypeError(PydanticCustomError):  # noqa F811
            ...

        class PydanticValueError(PydanticCustomError):  # noqa F811
            ...

    return PydanticTypeError, PydanticValueError



from typing import Any, Dict

from ..get_versions import get_pydantic_version

pydantic_version = get_pydantic_version()


if pydantic_version.value == 1:

    class core_schema:  # noqa
        def CoreSchema(self):
            ...

        def general_after_validator_function(self):
            ...

        def str_schema(self):
            ...

        def ValidationInfo(self):
            ...


if pydantic_version.value == 2:
    from pydantic_core import core_schema  # noqa


JsonSchemaValue = Dict[str, Any]


class BasePydanticV2:
    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        source,
    ) -> core_schema.CoreSchema:
        return core_schema.general_after_validator_function(
            cls._validate, core_schema.str_schema()
        )

    @classmethod
    def __get_pydantic_json_schema__(
        cls, core_schema: core_schema.CoreSchema, handler
    ) -> JsonSchemaValue:
        field_schema = handler(core_schema)
        field_schema.update(type="string", format=cls.format)
        return field_schema

    @classmethod
    def _validate(cls, __input_value: str, _: core_schema.ValidationInfo) -> str:
        cls.validate_type(__input_value)
        return cls.validate(__input_value)


class BaseMaskV2(BasePydanticV2):
    @classmethod
    def _validate(cls, __input_value: str, _: core_schema.ValidationInfo) -> str:
        cls.validate_type(__input_value)
        cls.validate_mask(__input_value)
        return cls.validate(__input_value)


class BaseDigitsV2(BasePydanticV2):
    @classmethod
    def _validate(cls, __input_value: str, _: core_schema.ValidationInfo) -> str:
        cls.validate_type(__input_value)
        cls.validate_numbers(__input_value)
        return cls.validate(__input_value)



from ..validators.cep_validator import CEPValidator
from .base_field import Base, BaseDigits, BaseMask

__all__ = [
    "CEP",
    "CEPMask",
    "CEPDigits",
]


class CEP(Base):
    """
    Only Accepts string of CEP.

    Attributes:
        number (str): CEP number.

    """

    format = "cep"
    Validator = CEPValidator


class CEPMask(BaseMask):
    """
    Only Accepts string of CEP with mask.

    Attributes:
        number (str): CEP number.
    """

    format = "cep"
    Validator = CEPValidator


class CEPDigits(BaseDigits):
    """
    Only Accepts string of CEP without mask.

    Attributes:
        number (str): CEP number.
    """

    format = "cep"
    Validator = CEPValidator
